@@ -1,32 +1,20 @@
-/* This file is part of KDE
-    Copyright (c) 2006-2016 David Faure <faure@kde.org>
+/*
+    This file is part of KDE
+    SPDX-FileCopyrightText: 2006-2016 David Faure <faure@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-#include <qtest.h>
-#include <QStandardPaths>
-#include <KIO/Job>
 #include <KIO/FavIconRequestJob>
+#include <KIO/Job>
 #include <QDir>
 #include <QLoggingCategory>
+#include <QStandardPaths>
+#include <QTest>
 
 #include <QFutureSynchronizer>
-#include <QtConcurrentRun>
 #include <QThreadPool>
+#include <QtConcurrentRun>
 
 static const char s_hostUrl[] = "http://www.google.com/index.html";
 static const char s_pageUrl[] = "http://www.google.com/somepage.html";
@@ -92,7 +80,7 @@ void FavIconTest::initTestCase()
     QVERIFY(!QFileInfo::exists(favIconCacheDir));
 
     // Enable debug output
-    QLoggingCategory::setFilterRules(QStringLiteral("kde.kio.favicons.debug=true"));
+    QLoggingCategory::setFilterRules(QStringLiteral("kf.kio.gui.favicons.debug=true"));
 }
 
 void FavIconTest::favIconForUrlShouldBeEmptyInitially()
@@ -119,7 +107,7 @@ void FavIconTest::hostJobShouldDownloadIconThenUseCache()
     QVERIFY2(QFile::exists(iconFile), qPrintable(iconFile));
     QVERIFY(!QIcon(iconFile).isNull()); // pass full path to QIcon
     // This requires https://codereview.qt-project.org/148444
-    //QVERIFY(!QIcon::fromTheme(iconFile).isNull()); // old code ported from kdelibs4 might do that, should work too
+    // QVERIFY(!QIcon::fromTheme(iconFile).isNull()); // old code ported from kdelibs4 might do that, should work too
 
     // Lookup should give the same result
     QCOMPARE(KIO::favIconForUrl(url), iconFile);
@@ -133,13 +121,13 @@ void FavIconTest::hostJobShouldDownloadIconThenUseCache()
     // The code from the class docu
     QString goticonFile;
     {
-        KIO::FavIconRequestJob *job = new KIO::FavIconRequestJob(url);
-        connect(job, &KIO::FavIconRequestJob::result, this, [job, &goticonFile](KJob *){
-            if (!job->error()) {
-                goticonFile = job->iconFile();
+        KIO::FavIconRequestJob *favJob = new KIO::FavIconRequestJob(url);
+        connect(favJob, &KIO::FavIconRequestJob::result, this, [favJob, &goticonFile](KJob *) {
+            if (!favJob->error()) {
+                goticonFile = favJob->iconFile();
             }
         });
-        QVERIFY(job->exec());
+        QVERIFY(favJob->exec());
     }
     QCOMPARE(goticonFile, iconFile);
 }
@@ -233,7 +221,7 @@ void FavIconTest::tooBigFaviconShouldAbort()
     job->setIconUrl(QUrl("http://download.kde.org/Attic/4.13.2/src/kcalc-4.13.2.tar.xz"));
     QVERIFY(willDownload(job));
     QVERIFY(!job->exec());
-    QCOMPARE(job->error(), int(KIO::ERR_SLAVE_DEFINED));
+    QCOMPARE(job->error(), int(KIO::ERR_WORKER_DEFINED));
     QCOMPARE(job->errorString(), QStringLiteral("Icon file too big, download aborted"));
 }
 
