@@ -7,7 +7,7 @@
 // This test suite is based on those in ftptest.cpp and uses the same test files.
 
 #include <kio/copyjob.h>
-#include <kio/job.h>
+#include <kio/storedtransferjob.h>
 
 #include <QBuffer>
 #include <QProcess>
@@ -36,7 +36,11 @@ private:
     static void runDaemon(QProcess &proc, const QTemporaryDir &remoteDir)
     {
         QVERIFY(remoteDir.isValid());
-        proc.setProgram("wsgidav");
+        const QString exec = QStandardPaths::findExecutable("wsgidav");
+        if (exec.isEmpty()) {
+            QFAIL("Could not find 'wsgidav' executable in PATH");
+        }
+        proc.setProgram(exec);
         proc.setArguments(
             {QStringLiteral("--host=0.0.0.0"), QString("--port=%1").arg(port), QString("--root=%1").arg(remoteDir.path()), QStringLiteral("--auth=anonymous")});
         proc.setProcessChannelMode(QProcess::ForwardedErrorChannel);
@@ -207,7 +211,7 @@ private Q_SLOTS:
 
     void testOverwriteCopyWithoutFlagFromRemote()
     {
-        // This excercises a different code path than testOverwriteCopyWithoutFlagFromLocal
+        // This exercises a different code path than testOverwriteCopyWithoutFlagFromLocal
         const QString path("/testOverwriteCopyWithoutFlagRemote");
         const QString dir_path("/dir");
         const auto url = this->url(path);
