@@ -113,6 +113,13 @@ public:
         Delete, /// Delete the files/directories directly, i.e. without moving them to Trash
         Trash, /// Move the files/directories to Trash
         EmptyTrash, /// Empty the Trash
+        /**
+         * This is the same as Delete, but more text is added to the message to inform
+         * the user that moving to Trash was tried but failed due to size constraints.
+         * Typical use case is re-asking the user about deleting instead of Trashing.
+         * @since 5.100
+         */
+        DeleteInsteadOfTrash,
     };
 
     /**
@@ -150,10 +157,10 @@ public:
                                QWidget *parent = nullptr) = 0; // TODO KF6: replace QWidget* with QObject*
 
     enum MessageDialogType {
-        QuestionYesNo = 1,
-        QuestionYesNoCancel = 2,
-        WarningYesNo = 3,
-        WarningYesNoCancel = 4,
+        QuestionTwoActions = 1, ///< @since 5.100
+        QuestionTwoActionsCancel = 2, ///< @since 5.100
+        WarningTwoActions = 3, ///< @since 5.100
+        WarningTwoActionsCancel = 4, ///< @since 5.100
         WarningContinueCancel = 5,
         SSLMessageBox = 6,
         Information = 7,
@@ -162,6 +169,16 @@ public:
             KIOCORE_ENUMERATOR_DEPRECATED_VERSION(5, 97, "Use Error.") = 8,
 #endif
         Error = 9,
+#if KIOCORE_ENABLE_DEPRECATED_SINCE(5, 100)
+        QuestionYesNo ///< @deprecated Since 5.100, use QuestionTwoActions.
+            KIOCORE_ENUMERATOR_DEPRECATED_VERSION(5, 100, "Use QuestionTwoActions.") = QuestionTwoActions,
+        QuestionYesNoCancel ///< @deprecated Since 5.100, use QuestionTwoActionsCancel.
+            KIOCORE_ENUMERATOR_DEPRECATED_VERSION(5, 100, "Use QuestionTwoActionsCancel.") = QuestionTwoActionsCancel,
+        WarningYesNo ///< @deprecated Since 5.100, use WarningTwoActions.
+            KIOCORE_ENUMERATOR_DEPRECATED_VERSION(5, 100, "Use WarningTwoActions.") = WarningTwoActions,
+        WarningYesNoCancel ///< @deprecated Since 5.100, use WarningTwoActionsCancel.
+            KIOCORE_ENUMERATOR_DEPRECATED_VERSION(5, 100, "Use WarningTwoActionsCancel.") = WarningTwoActionsCancel,
+#endif
     };
 
     /**
@@ -170,10 +187,10 @@ public:
      * @param type the desired type of message box, see the MessageDialogType enum
      * @param text the message to show to the user
      * @param title the title of the message dialog box
-     * @param buttonYes the text for the YES button
-     * @param buttonNo the text for the NO button
-     * @param iconYes the icon to show on the YES button
-     * @param iconNo the icon to show on the NO button
+     * @param primaryActionText the text for the primary action
+     * @param secondatyActionText the text for the secondary action
+     * @param primaryActionIconName the icon to show on the primary action
+     * @param secondatyActionIconName the icon to show on the secondary action
      * @param dontAskAgainName the config key name used to store the result from
      *                     'Do not ask again' checkbox
      * @param details more details about the message shown to the user
@@ -183,10 +200,10 @@ public:
     virtual void requestUserMessageBox(MessageDialogType type,
                                        const QString &text,
                                        const QString &title,
-                                       const QString &buttonYes,
-                                       const QString &buttonNo,
-                                       const QString &iconYes = {},
-                                       const QString &iconNo = {},
+                                       const QString &primaryActionText,
+                                       const QString &secondatyActionText,
+                                       const QString &primaryActionIconName = {},
+                                       const QString &secondatyActionIconName = {},
                                        const QString &dontAskAgainName = {},
                                        const QString &details = {},
                                        const KIO::MetaData &sslMetaData = {},
