@@ -12,71 +12,86 @@
 
 #include <config-kiocore.h>
 
-#include <stdlib.h>
 #include <qplatformdefs.h>
 #include <signal.h>
+#include <stdlib.h>
 #ifdef Q_OS_WIN
 #include <process.h>
 #endif
 
-#include <QtGlobal>
-#include <QFile>
-#include <QList>
-#include <QDateTime>
-#include <QElapsedTimer>
 #include <QCoreApplication>
 #include <QDataStream>
+#include <QDateTime>
+#include <QElapsedTimer>
+#include <QFile>
+#include <QList>
 #include <QMap>
+#include <QtGlobal>
 
 #include <KConfig>
 #include <KConfigGroup>
-#include <KCrash>
 #include <KLocalizedString>
+#include <QThread>
+
+#ifndef Q_OS_ANDROID
+#include <KCrash>
+#endif
 
 #include "kremoteencoding.h"
 
-#include "kioglobal_p.h"
-#include "connection_p.h"
 #include "commands_p.h"
-#include "ioslave_defaults.h"
-#include "slaveinterface.h"
-#include "kpasswdserverclient.h"
+#include "connection_p.h"
+#include "ioworker_defaults.h"
 #include "kiocoredebug.h"
+#include "kioglobal_p.h"
+#include "kpasswdserverclient.h"
+#include "slaveinterface.h"
 
-#ifdef Q_OS_UNIX
-#include <KAuth>
+#if defined(Q_OS_UNIX) && !defined(Q_OS_ANDROID)
+#include <KAuth/Action>
 #endif
 
 #if KIO_ASSERT_SLAVE_STATES
 #define KIO_STATE_ASSERT(cond, where, what) Q_ASSERT_X(cond, where, what)
 #else
-#define KIO_STATE_ASSERT(cond, where, what) do { if (!(cond)) qCWarning(KIO_CORE) << what; } while (false)
+/* clang-format off */
+#define KIO_STATE_ASSERT(cond, where, what) \
+    do { \
+        if (!(cond)) { \
+            qCWarning(KIO_CORE) << what; \
+        } \
+    } while (false)
 #endif
+/* clang-format on */
 
 extern "C" {
-    static void sigpipe_handler(int sig);
+static void sigpipe_handler(int sig);
 }
 
 using namespace KIO;
 
 typedef QList<QByteArray> AuthKeysList;
 typedef QMap<QString, QByteArray> AuthKeysMap;
-#define KIO_DATA QByteArray data; QDataStream stream( &data, QIODevice::WriteOnly ); stream
-#define KIO_FILESIZE_T(x) quint64(x)
-static const int KIO_MAX_ENTRIES_PER_BATCH = 200;
-static const int KIO_MAX_SEND_BATCH_TIME = 300;
+
+/* clang-format off */
+#define KIO_DATA \
+    QByteArray data; \
+    QDataStream stream(&data, QIODevice::WriteOnly); \
+    stream
+/* clang-format on */
+
+static constexpr int KIO_MAX_ENTRIES_PER_BATCH = 200;
+static constexpr int KIO_MAX_SEND_BATCH_TIME = 300;
 
 namespace KIO
 {
-
 class SlaveBasePrivate
 {
 public:
-    SlaveBase * const q;
+    SlaveBase *const q;
     explicit SlaveBasePrivate(SlaveBase *owner)
         : q(owner)
         , nextTimeoutMsecs(0)
-        , m_passwdServerClient(nullptr)
         , m_confirmationAsked(false)
         , m_privilegeOperationStatus(OperationNotAllowed)
     {
@@ -86,10 +101,7 @@ public:
         pendingListEntries.reserve(KIO_MAX_ENTRIES_PER_BATCH);
         appConnection.setReadMode(Connection::ReadMode::Polled);
     }
-    ~SlaveBasePrivate()
-    {
-        delete m_passwdServerClient;
-    }
+    ~SlaveBasePrivate() = default;
 
     UDSEntryList pendingListEntries;
     QElapsedTimer m_timeSinceLastBatch;
@@ -98,12 +110,13 @@ public:
     bool isConnectedToApp;
 
     QString slaveid;
-    bool resume: 1;
-    bool needSendCanResume: 1;
-    bool onHold: 1;
-    bool wasKilled: 1;
-    bool inOpenLoop: 1;
-    bool exit_loop: 1;
+    bool resume : 1;
+    bool needSendCanResume : 1;
+    bool onHold : 1;
+    bool inOpenLoop : 1;
+    std::atomic<bool> wasKilled = false;
+    std::atomic<bool> exit_loop = false;
+    std::atomic<bool> runInThread = false;
     MetaData configData;
     KConfig *config = nullptr;
     KConfigGroup *configGroup = nullptr;
@@ -119,18 +132,20 @@ public:
     bool m_finalityCommand = true; // whether finished() or error() may/must be called
     QByteArray timeoutData;
 
-    KPasswdServerClient *m_passwdServerClient = nullptr;
+#ifndef KIO_ANDROID_STUB
+    std::unique_ptr<KPasswdServerClient> m_passwdServerClient;
+#endif
     bool m_rootEntryListed = false;
 
     bool m_confirmationAsked;
     QSet<QString> m_tempAuths;
-    QString m_warningCaption;
+    QString m_warningTitle;
     QString m_warningMessage;
     int m_privilegeOperationStatus;
 
     void updateTempAuthStatus()
     {
-#ifdef Q_OS_UNIX
+#if defined(Q_OS_UNIX) && !defined(Q_OS_ANDROID)
         QSet<QString>::iterator it = m_tempAuths.begin();
         while (it != m_tempAuths.end()) {
             KAuth::Action action(*it);
@@ -181,8 +196,8 @@ public:
         KIO_STATE_ASSERT(finalState(),
                          Q_FUNC_INFO,
                          qUtf8Printable(QStringLiteral("%1 did not call finished() or error()! Please fix the %2 KIO slave")
-                                        .arg(QLatin1String(cmdName))
-                                        .arg(QCoreApplication::applicationName())));
+                                            .arg(QLatin1String(cmdName))
+                                            .arg(QCoreApplication::applicationName())));
         // Force the command into finished state. We'll not reach this for Debug builds
         // that fail the assertion. For Release builds we'll have made sure that the
         // command is actually finished after the verification regardless of what
@@ -197,75 +212,76 @@ public:
         KIO_STATE_ASSERT(!finalState(),
                          Q_FUNC_INFO,
                          qUtf8Printable(QStringLiteral("%1 called finished() or error(), but it's not supposed to! Please fix the %2 KIO slave")
-                                        .arg(QLatin1String(cmdName))
-                                        .arg(QCoreApplication::applicationName())));
+                                            .arg(QLatin1String(cmdName))
+                                            .arg(QCoreApplication::applicationName())));
     }
 
+#ifndef KIO_ANDROID_STUB
     KPasswdServerClient *passwdServerClient()
     {
         if (!m_passwdServerClient) {
-            m_passwdServerClient = new KPasswdServerClient;
+            m_passwdServerClient = std::make_unique<KPasswdServerClient>();
         }
 
-        return m_passwdServerClient;
+        return m_passwdServerClient.get();
     }
+#endif
 };
 
 }
 
-static SlaveBase *globalSlave;
-
 static volatile bool slaveWriteError = false;
 
-static const char *s_protocol;
-
 #ifdef Q_OS_UNIX
+static SlaveBase *globalSlave;
+
 extern "C" {
-    static void genericsig_handler(int sigNumber)
-    {
-        ::signal(sigNumber, SIG_IGN);
-        //WABA: Don't do anything that requires malloc, we can deadlock on it since
-        //a SIGTERM signal can come in while we are in malloc/free.
-        //qDebug()<<"kioslave : exiting due to signal "<<sigNumber;
-        //set the flag which will be checked in dispatchLoop() and which *should* be checked
-        //in lengthy operations in the various slaves
-        if (globalSlave != nullptr) {
-            globalSlave->setKillFlag();
-        }
-        ::signal(SIGALRM, SIG_DFL);
-        alarm(5);  //generate an alarm signal in 5 seconds, in this time the slave has to exit
+static void genericsig_handler(int sigNumber)
+{
+    ::signal(sigNumber, SIG_IGN);
+    // WABA: Don't do anything that requires malloc, we can deadlock on it since
+    // a SIGTERM signal can come in while we are in malloc/free.
+    // qDebug()<<"kioslave : exiting due to signal "<<sigNumber;
+    // set the flag which will be checked in dispatchLoop() and which *should* be checked
+    // in lengthy operations in the various slaves
+    if (globalSlave != nullptr) {
+        globalSlave->setKillFlag();
     }
+    ::signal(SIGALRM, SIG_DFL);
+    alarm(5); // generate an alarm signal in 5 seconds, in this time the slave has to exit
+}
 }
 #endif
 
 //////////////
 
-SlaveBase::SlaveBase(const QByteArray &protocol,
-                     const QByteArray &pool_socket,
-                     const QByteArray &app_socket)
-    : mProtocol(protocol),
-      d(new SlaveBasePrivate(this))
+SlaveBase::SlaveBase(const QByteArray &protocol, const QByteArray &pool_socket, const QByteArray &app_socket)
+    : mProtocol(protocol)
+    , d(new SlaveBasePrivate(this))
 
 {
     Q_ASSERT(!app_socket.isEmpty());
     d->poolSocket = QFile::decodeName(pool_socket);
-    s_protocol = protocol.data();
 
-    KCrash::initialize();
-
-#ifdef Q_OS_UNIX
-    struct sigaction act;
-    act.sa_handler = sigpipe_handler;
-    sigemptyset(&act.sa_mask);
-    act.sa_flags = 0;
-    sigaction(SIGPIPE, &act, nullptr);
-
-    ::signal(SIGINT, &genericsig_handler);
-    ::signal(SIGQUIT, &genericsig_handler);
-    ::signal(SIGTERM, &genericsig_handler);
+    if (QThread::currentThread() == qApp->thread()) {
+#ifndef Q_OS_ANDROID
+        KCrash::initialize();
 #endif
 
-    globalSlave = this;
+#ifdef Q_OS_UNIX
+        struct sigaction act;
+        act.sa_handler = sigpipe_handler;
+        sigemptyset(&act.sa_mask);
+        act.sa_flags = 0;
+        sigaction(SIGPIPE, &act, nullptr);
+
+        ::signal(SIGINT, &genericsig_handler);
+        ::signal(SIGQUIT, &genericsig_handler);
+        ::signal(SIGTERM, &genericsig_handler);
+
+        globalSlave = this;
+#endif
+    }
 
     d->isConnectedToApp = true;
 
@@ -275,14 +291,12 @@ SlaveBase::SlaveBase(const QByteArray &protocol,
     d->needSendCanResume = false;
     d->mapConfig = QMap<QString, QVariant>();
     d->onHold = false;
-    d->wasKilled = false;
-//    d->processed_size = 0;
+    //    d->processed_size = 0;
     d->totalSize = 0;
     connectSlave(QFile::decodeName(app_socket));
 
     d->remotefile = nullptr;
     d->inOpenLoop = false;
-    d->exit_loop = false;
 }
 
 SlaveBase::~SlaveBase()
@@ -290,8 +304,6 @@ SlaveBase::~SlaveBase()
     delete d->configGroup;
     delete d->config;
     delete d->remotefile;
-    delete d;
-    s_protocol = "";
 }
 
 void SlaveBase::dispatchLoop()
@@ -302,6 +314,8 @@ void SlaveBase::dispatchLoop()
             d->nextTimeout.invalidate();
             d->timeoutData = QByteArray();
             special(data);
+            // NB: not performing state validation since http slave is misbehaving and not emitting anything at the time of writing - August 2022
+            d->m_state = d->Idle;
         }
 
         Q_ASSERT(d->appConnection.inited());
@@ -342,9 +356,9 @@ void SlaveBase::dispatchLoop()
             }
         }
 
-        //I think we get here when we were killed in dispatch() and not in select()
+        // I think we get here when we were killed in dispatch() and not in select()
         if (wasKilled()) {
-            //qDebug() << "slave was killed, returning";
+            // qDebug() << "slave was killed, returning";
             break;
         }
 
@@ -432,8 +446,7 @@ KConfigGroup *SlaveBase::config()
         d->configGroup = new KConfigGroup(d->config, QString());
 
         auto end = d->mapConfig.cend();
-        for (auto it = d->mapConfig.cbegin(); it != end; ++it)
-        {
+        for (auto it = d->mapConfig.cbegin(); it != end; ++it) {
             d->configGroup->writeEntry(it.key(), it->toString().toUtf8(), KConfigGroup::WriteConfigFlags());
         }
     }
@@ -474,7 +487,7 @@ void SlaveBase::data(const QByteArray &data)
 
 void SlaveBase::dataReq()
 {
-    //sendMetaData();
+    // sendMetaData();
     if (d->needSendCanResume) {
         canResume(0);
     }
@@ -490,22 +503,21 @@ void SlaveBase::opened()
 
 void SlaveBase::error(int _errid, const QString &_text)
 {
-    KIO_STATE_ASSERT(d->m_finalityCommand,
-                     Q_FUNC_INFO,
-                     qUtf8Printable(QStringLiteral("error() was called, but it's not supposed to! Please fix the %1 KIO slave")
-                                    .arg(QCoreApplication::applicationName())));
+    KIO_STATE_ASSERT(
+        d->m_finalityCommand,
+        Q_FUNC_INFO,
+        qUtf8Printable(QStringLiteral("error() was called, but it's not supposed to! Please fix the %1 KIO slave").arg(QCoreApplication::applicationName())));
 
     if (d->m_state == d->ErrorCalled) {
         KIO_STATE_ASSERT(false,
                          Q_FUNC_INFO,
-                         qUtf8Printable(QStringLiteral("error() called twice! Please fix the %1 KIO slave")
-                                        .arg(QCoreApplication::applicationName())));
+                         qUtf8Printable(QStringLiteral("error() called twice! Please fix the %1 KIO slave").arg(QCoreApplication::applicationName())));
         return;
     } else if (d->m_state == d->FinishedCalled) {
-        KIO_STATE_ASSERT(false,
-                         Q_FUNC_INFO,
-                         qUtf8Printable(QStringLiteral("error() called after finished()! Please fix the %1 KIO slave")
-                                        .arg(QCoreApplication::applicationName())));
+        KIO_STATE_ASSERT(
+            false,
+            Q_FUNC_INFO,
+            qUtf8Printable(QStringLiteral("error() called after finished()! Please fix the %1 KIO slave").arg(QCoreApplication::applicationName())));
         return;
     }
 
@@ -516,7 +528,7 @@ void SlaveBase::error(int _errid, const QString &_text)
     KIO_DATA << static_cast<qint32>(_errid) << _text;
 
     send(MSG_ERROR, data);
-    //reset
+    // reset
     d->totalSize = 0;
     d->inOpenLoop = false;
     d->m_confirmationAsked = false;
@@ -546,22 +558,22 @@ void SlaveBase::finished()
         d->pendingListEntries.clear();
     }
 
-    KIO_STATE_ASSERT(d->m_finalityCommand,
-                     Q_FUNC_INFO,
-                     qUtf8Printable(QStringLiteral("finished() was called, but it's not supposed to! Please fix the %2 KIO slave")
-                                    .arg(QCoreApplication::applicationName())));
+    KIO_STATE_ASSERT(
+        d->m_finalityCommand,
+        Q_FUNC_INFO,
+        qUtf8Printable(
+            QStringLiteral("finished() was called, but it's not supposed to! Please fix the %2 KIO slave").arg(QCoreApplication::applicationName())));
 
     if (d->m_state == d->FinishedCalled) {
         KIO_STATE_ASSERT(false,
                          Q_FUNC_INFO,
-                         qUtf8Printable(QStringLiteral("finished() called twice! Please fix the %1 KIO slave")
-                                        .arg(QCoreApplication::applicationName())));
+                         qUtf8Printable(QStringLiteral("finished() called twice! Please fix the %1 KIO slave").arg(QCoreApplication::applicationName())));
         return;
     } else if (d->m_state == d->ErrorCalled) {
-        KIO_STATE_ASSERT(false,
-                         Q_FUNC_INFO,
-                         qUtf8Printable(QStringLiteral("finished() called after error()! Please fix the %1 KIO slave")
-                                        .arg(QCoreApplication::applicationName())));
+        KIO_STATE_ASSERT(
+            false,
+            Q_FUNC_INFO,
+            qUtf8Printable(QStringLiteral("finished() called after error()! Please fix the %1 KIO slave").arg(QCoreApplication::applicationName())));
         return;
     }
 
@@ -599,10 +611,10 @@ void SlaveBase::canResume()
 
 void SlaveBase::totalSize(KIO::filesize_t _bytes)
 {
-    KIO_DATA << KIO_FILESIZE_T(_bytes);
+    KIO_DATA << static_cast<quint64>(_bytes);
     send(INF_TOTAL_SIZE, data);
 
-    //this one is usually called before the first item is listed in listDir()
+    // this one is usually called before the first item is listed in listDir()
     d->totalSize = _bytes;
 }
 
@@ -621,7 +633,7 @@ void SlaveBase::processedSize(KIO::filesize_t _bytes)
     }
 
     if (emitSignal) {
-        KIO_DATA << KIO_FILESIZE_T(_bytes);
+        KIO_DATA << static_cast<quint64>(_bytes);
         send(INF_PROCESSED_SIZE, data);
         d->lastTimeout.start();
     }
@@ -631,25 +643,25 @@ void SlaveBase::processedSize(KIO::filesize_t _bytes)
 
 void SlaveBase::written(KIO::filesize_t _bytes)
 {
-    KIO_DATA << KIO_FILESIZE_T(_bytes);
+    KIO_DATA << static_cast<quint64>(_bytes);
     send(MSG_WRITTEN, data);
 }
 
 void SlaveBase::position(KIO::filesize_t _pos)
 {
-    KIO_DATA << KIO_FILESIZE_T(_pos);
+    KIO_DATA << static_cast<quint64>(_pos);
     send(INF_POSITION, data);
 }
 
 void SlaveBase::truncated(KIO::filesize_t _length)
 {
-    KIO_DATA << KIO_FILESIZE_T(_length);
+    KIO_DATA << static_cast<quint64>(_length);
     send(INF_TRUNCATED, data);
 }
 
 void SlaveBase::processedPercent(float /* percent */)
 {
-    //qDebug() << "STUB";
+    // qDebug() << "STUB";
 }
 
 void SlaveBase::speed(unsigned long _bytes_per_second)
@@ -671,24 +683,26 @@ void SlaveBase::errorPage()
 
 static bool isSubCommand(int cmd)
 {
-    return ((cmd == CMD_REPARSECONFIGURATION) ||
-            (cmd == CMD_META_DATA) ||
-            (cmd == CMD_CONFIG) ||
-            (cmd == CMD_SUBURL) ||
-            (cmd == CMD_SLAVE_STATUS) ||
-            (cmd == CMD_SLAVE_CONNECT) ||
-            (cmd == CMD_SLAVE_HOLD) ||
-            (cmd == CMD_MULTI_GET));
+    /* clang-format off */
+    return cmd == CMD_REPARSECONFIGURATION
+        || cmd == CMD_META_DATA
+        || cmd == CMD_CONFIG
+        || cmd == CMD_SUBURL
+        || cmd == CMD_SLAVE_STATUS
+        || cmd == CMD_SLAVE_CONNECT
+        || cmd == CMD_SLAVE_HOLD
+        || cmd == CMD_MULTI_GET;
+    /* clang-format on */
 }
 
 void SlaveBase::mimeType(const QString &_type)
 {
-    //qDebug() << _type;
+    // qDebug() << _type;
     int cmd;
     do {
-        // Send the meta-data each time we send the mime-type.
+        // Send the meta-data each time we send the MIME type.
         if (!mOutgoingMetaData.isEmpty()) {
-            //qDebug() << "emitting meta data";
+            // qDebug() << "emitting meta data";
             KIO_DATA << mOutgoingMetaData;
             send(INF_META_DATA, data);
         }
@@ -701,10 +715,10 @@ void SlaveBase::mimeType(const QString &_type)
                 ret = d->appConnection.read(&cmd, data);
             }
             if (ret == -1) {
-                //qDebug() << "read error";
+                // qDebug() << "read error";
                 exit();
             }
-            //qDebug() << "got" << cmd;
+            // qDebug() << "got" << cmd;
             if (cmd == CMD_HOST) { // Ignore.
                 continue;
             }
@@ -718,14 +732,18 @@ void SlaveBase::mimeType(const QString &_type)
     mOutgoingMetaData.clear();
 }
 
-void SlaveBase::exit()
+void SlaveBase::exit() // possibly called from another thread, only use atomics in here
 {
     d->exit_loop = true;
-    // Using ::exit() here is too much (crashes in qdbus's qglobalstatic object),
-    // so let's cleanly exit dispatchLoop() instead.
-    // Update: we do need to call exit(), otherwise a long download (get()) would
-    // keep going until it ends, even though the application exited.
-    ::exit(255);
+    if (d->runInThread) {
+        d->wasKilled = true;
+    } else {
+        // Using ::exit() here is too much (crashes in qdbus's qglobalstatic object),
+        // so let's cleanly exit dispatchLoop() instead.
+        // Update: we do need to call exit(), otherwise a long download (get()) would
+        // keep going until it ends, even though the application exited.
+        ::exit(255);
+    }
 }
 
 void SlaveBase::warning(const QString &_msg)
@@ -882,7 +900,7 @@ KIOCORE_EXPORT QString KIO::unsupportedActionErrorString(const QString &protocol
         return i18n("Opening files is not supported with protocol %1.", protocol);
     default:
         return i18n("Protocol %1 does not support action %2.", protocol, cmd);
-    }/*end switch*/
+    } /*end switch*/
 }
 
 void SlaveBase::openConnection()
@@ -890,7 +908,8 @@ void SlaveBase::openConnection()
     error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_CONNECT));
 }
 void SlaveBase::closeConnection()
-{ } // No response!
+{
+} // No response!
 void SlaveBase::stat(QUrl const &)
 {
     error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_STAT));
@@ -1018,50 +1037,72 @@ int SlaveBase::openPasswordDialogV2(AuthInfo &info, const QString &errorMsg)
     // it to ensure it is valid.
     dlgInfo.setExtraField(QStringLiteral("skip-caching-on-query"), true);
 
+#ifndef KIO_ANDROID_STUB
     KPasswdServerClient *passwdServerClient = d->passwdServerClient();
     const int errCode = passwdServerClient->queryAuthInfo(&dlgInfo, errorMessage, windowId, userTimestamp);
     if (errCode == KJob::NoError) {
         info = dlgInfo;
     }
     return errCode;
+#else
+    return KJob::NoError;
+#endif
 }
 
-int SlaveBase::messageBox(MessageBoxType type, const QString &text, const QString &caption,
-                          const QString &buttonYes, const QString &buttonNo)
+int SlaveBase::messageBox(MessageBoxType type, const QString &text, const QString &title, const QString &primaryActionText, const QString &secondaryActionText)
 {
-    return messageBox(text, type, caption, buttonYes, buttonNo, QString());
+    return messageBox(text, type, title, primaryActionText, secondaryActionText, QString());
 }
 
-int SlaveBase::messageBox(const QString &text, MessageBoxType type, const QString &caption,
-                          const QString &_buttonYes, const QString &_buttonNo,
+int SlaveBase::messageBox(const QString &text,
+                          MessageBoxType type,
+                          const QString &title,
+                          const QString &primaryActionText,
+                          const QString &secondaryActionText,
                           const QString &dontAskAgainName)
 {
-    QString buttonYes = _buttonYes.isNull() ? i18n("&Yes") : _buttonYes;
-    QString buttonNo = _buttonNo.isNull() ? i18n("&No") : _buttonNo;
-    //qDebug() << "messageBox " << type << " " << text << " - " << caption << buttonYes << buttonNo;
-    KIO_DATA << static_cast<qint32>(type) << text << caption << buttonYes << buttonNo << dontAskAgainName;
+#if KIOCORE_BUILD_DEPRECATED_SINCE(5, 100)
+    if ((type != Information) || (type != SSLMessageBox)) {
+        if (primaryActionText.isNull()) {
+            qCWarning(KIO_CORE) << "Deprecated: messageBox() called with null primaryActionText arg."
+                                << "type:" << type << "text:" << text;
+        }
+        if ((type != WarningContinueCancel) || (type != WarningContinueCancelDetailed)) {
+            if (secondaryActionText.isNull()) {
+                qCWarning(KIO_CORE) << "Deprecated: messageBox() called with null secondaryActionText arg."
+                                    << "type:" << type << "text:" << text;
+            }
+        }
+    }
+    const QString _primaryActionText = primaryActionText.isNull() ? i18n("&Yes") : primaryActionText;
+    const QString _secondaryActionText = secondaryActionText.isNull() ? i18n("&No") : secondaryActionText;
+    // qDebug() << "messageBox " << type << " " << text << " - " << title << _primaryActionText << _secondaryActionText;
+    KIO_DATA << static_cast<qint32>(type) << text << title << _primaryActionText << _secondaryActionText << dontAskAgainName;
+#else
+    KIO_DATA << static_cast<qint32>(type) << text << title << primaryActionText << secondaryActionText << dontAskAgainName;
+#endif
     send(INF_MESSAGEBOX, data);
     if (waitForAnswer(CMD_MESSAGEBOXANSWER, 0, data) != -1) {
         QDataStream stream(data);
         int answer;
         stream >> answer;
-        //qDebug() << "got messagebox answer" << answer;
+        // qDebug() << "got messagebox answer" << answer;
         return answer;
     } else {
-        return 0;    // communication failure
+        return 0; // communication failure
     }
 }
 
 bool SlaveBase::canResume(KIO::filesize_t offset)
 {
-    //qDebug() << "offset=" << KIO::number(offset);
+    // qDebug() << "offset=" << KIO::number(offset);
     d->needSendCanResume = false;
-    KIO_DATA << KIO_FILESIZE_T(offset);
+    KIO_DATA << static_cast<quint64>(offset);
     send(MSG_RESUME, data);
     if (offset) {
         int cmd;
         if (waitForAnswer(CMD_RESUMEANSWER, CMD_NONE, data, &cmd) != -1) {
-            //qDebug() << "returning" << (cmd == CMD_RESUMEANSWER);
+            // qDebug() << "returning" << (cmd == CMD_RESUMEANSWER);
             return cmd == CMD_RESUMEANSWER;
         } else {
             return false;
@@ -1080,7 +1121,7 @@ int SlaveBase::waitForAnswer(int expected1, int expected2, QByteArray &data, int
             result = d->appConnection.read(&cmd, data);
         }
         if (result == -1) {
-            //qDebug() << "read error.";
+            // qDebug() << "read error.";
             return -1;
         }
 
@@ -1101,20 +1142,20 @@ int SlaveBase::waitForAnswer(int expected1, int expected2, QByteArray &data, int
 int SlaveBase::readData(QByteArray &buffer)
 {
     int result = waitForAnswer(MSG_DATA, 0, buffer);
-    //qDebug() << "readData: length = " << result << " ";
+    // qDebug() << "readData: length = " << result << " ";
     return result;
 }
 
 void SlaveBase::setTimeoutSpecialCommand(int timeout, const QByteArray &data)
 {
     if (timeout > 0) {
-        d->nextTimeoutMsecs = timeout*1000; // from seconds to milliseconds
+        d->nextTimeoutMsecs = timeout * 1000; // from seconds to milliseconds
         d->nextTimeout.start();
     } else if (timeout == 0) {
-        d->nextTimeoutMsecs = 1000;  // Immediate timeout
+        d->nextTimeoutMsecs = 1000; // Immediate timeout
         d->nextTimeout.start();
     } else {
-        d->nextTimeout.invalidate();  // Canceled
+        d->nextTimeout.invalidate(); // Canceled
     }
 
     d->timeoutData = data;
@@ -1132,27 +1173,32 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
     switch (command) {
     case CMD_HOST: {
         QString passwd;
-        QString host, user;
+        QString host;
+        QString user;
         quint16 port;
         stream >> host >> port >> user >> passwd;
         d->m_state = d->InsideMethod;
         d->m_finalityCommand = false;
         setHost(host, port, user, passwd);
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_CONNECT: {
         openConnection();
-    } break;
+        break;
+    }
     case CMD_DISCONNECT: {
         closeConnection();
-    } break;
+        break;
+    }
     case CMD_SLAVE_STATUS: {
         d->m_state = d->InsideMethod;
         d->m_finalityCommand = false;
         slave_status();
         // TODO verify that the slave has called slaveStatus()?
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_SLAVE_CONNECT: {
         d->onHold = false;
         QString app_socket;
@@ -1163,8 +1209,9 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         d->isConnectedToApp = true;
         connectSlave(app_socket);
         virtual_hook(AppConnectionMade, nullptr);
-    } break;
-    case CMD_SLAVE_HOLD: {
+        break;
+    }
+    case CMD_SLAVE_HOLD: { // TODO KF6: remove, unused
         QUrl url;
         QDataStream stream(data);
         stream >> url;
@@ -1172,38 +1219,46 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         d->onHold = true;
         disconnectSlave();
         d->isConnectedToApp = false;
-        // Do not close connection!
-        connectSlave(d->poolSocket);
-    } break;
+        if (!d->poolSocket.isEmpty()) {
+            // Do not close connection!
+            connectSlave(d->poolSocket);
+        }
+        break;
+    }
     case CMD_REPARSECONFIGURATION: {
         d->m_state = d->InsideMethod;
         d->m_finalityCommand = false;
         reparseConfiguration();
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_CONFIG: {
         stream >> d->configData;
         d->rebuildConfig();
         delete d->remotefile;
         d->remotefile = nullptr;
-    } break;
+        break;
+    }
     case CMD_GET: {
         stream >> url;
         d->m_state = d->InsideMethod;
         get(url);
         d->verifyState("get()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_OPEN: {
         stream >> url >> i;
         QIODevice::OpenMode mode = QFlag(i);
         d->m_state = d->InsideMethod;
-        open(url, mode); //krazy:exclude=syscalls
+        open(url, mode); // krazy:exclude=syscalls
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_PUT: {
         int permissions;
-        qint8 iOverwrite, iResume;
+        qint8 iOverwrite;
+        qint8 iResume;
         stream >> url >> iOverwrite >> iResume >> permissions;
         JobFlags flags;
         if (iOverwrite != 0) {
@@ -1216,41 +1271,46 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         // Remember that we need to send canResume(), TransferJob is expecting
         // it. Well, in theory this shouldn't be done if resume is true.
         //   (the resume bool is currently unused)
-        d->needSendCanResume = true   /* !resume */;
+        d->needSendCanResume = true /* !resume */;
 
         d->m_state = d->InsideMethod;
         put(url, permissions, flags);
         d->verifyState("put()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_STAT: {
         stream >> url;
         d->m_state = d->InsideMethod;
-        stat(url);   //krazy:exclude=syscalls
+        stat(url); // krazy:exclude=syscalls
         d->verifyState("stat()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_MIMETYPE: {
         stream >> url;
         d->m_state = d->InsideMethod;
         mimetype(url);
         d->verifyState("mimetype()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_LISTDIR: {
         stream >> url;
         d->m_state = d->InsideMethod;
         listDir(url);
         d->verifyState("listDir()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_MKDIR: {
         stream >> url >> i;
         d->m_state = d->InsideMethod;
-        mkdir(url, i);   //krazy:exclude=syscalls
+        mkdir(url, i); // krazy:exclude=syscalls
         d->verifyState("mkdir()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_RENAME: {
         qint8 iOverwrite;
         QUrl url2;
@@ -1260,10 +1320,11 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
             flags |= Overwrite;
         }
         d->m_state = d->InsideMethod;
-        rename(url, url2, flags);   //krazy:exclude=syscalls
+        rename(url, url2, flags); // krazy:exclude=syscalls
         d->verifyState("rename()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_SYMLINK: {
         qint8 iOverwrite;
         QString target;
@@ -1276,7 +1337,8 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         symlink(target, url, flags);
         d->verifyState("symlink()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_COPY: {
         int permissions;
         qint8 iOverwrite;
@@ -1290,7 +1352,8 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         copy(url, url2, permissions, flags);
         d->verifyState("copy()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_DEL: {
         qint8 isFile;
         stream >> url >> isFile;
@@ -1298,22 +1361,26 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         del(url, isFile != 0);
         d->verifyState("del()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_CHMOD: {
         stream >> url >> i;
         d->m_state = d->InsideMethod;
         chmod(url, i);
         d->verifyState("chmod()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_CHOWN: {
-        QString owner, group;
+        QString owner;
+        QString group;
         stream >> url >> owner >> group;
         d->m_state = d->InsideMethod;
         chown(url, owner, group);
         d->verifyState("chown()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_SETMODIFICATIONTIME: {
         QDateTime dt;
         stream >> url >> dt;
@@ -1321,34 +1388,40 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         setModificationTime(url, dt);
         d->verifyState("setModificationTime()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_SPECIAL: {
         d->m_state = d->InsideMethod;
         special(data);
         d->verifyState("special()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_META_DATA: {
-        //qDebug() << "(" << getpid() << ") Incoming meta-data...";
+        // qDebug() << "(" << getpid() << ") Incoming meta-data...";
         stream >> mIncomingMetaData;
         d->rebuildConfig();
-    } break;
+        break;
+    }
     case CMD_SUBURL: {
         stream >> url;
         d->m_state = d->InsideMethod;
         setSubUrl(url);
         d->verifyErrorFinishedNotCalled("setSubUrl()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_NONE: {
         qCWarning(KIO_CORE) << "Got unexpected CMD_NONE!";
-    } break;
+        break;
+    }
     case CMD_MULTI_GET: {
         d->m_state = d->InsideMethod;
         multiGet(data);
         d->verifyState("multiGet()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     case CMD_FILESYSTEMFREESPACE: {
         stream >> url;
 
@@ -1358,19 +1431,24 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         virtual_hook(GetFileSystemFreeSpace, data);
         d->verifyState("fileSystemFreeSpace()");
         d->m_state = d->Idle;
-    } break;
+        break;
+    }
     default: {
         // Some command we don't understand.
         // Just ignore it, it may come from some future version of KIO.
-    } break;
+        break;
+    }
     }
 }
 
 bool SlaveBase::checkCachedAuthentication(AuthInfo &info)
 {
+#ifndef KIO_ANDROID_STUB
     KPasswdServerClient *passwdServerClient = d->passwdServerClient();
-    return (passwdServerClient->checkAuthInfo(&info, metaData(QStringLiteral("window-id")).toLong(),
-                                        metaData(QStringLiteral("user-timestamp")).toULong()));
+    return (passwdServerClient->checkAuthInfo(&info, metaData(QStringLiteral("window-id")).toLong(), metaData(QStringLiteral("user-timestamp")).toULong()));
+#else
+    return false;
+#endif
 }
 
 void SlaveBase::dispatchOpenCommand(int command, const QByteArray &data)
@@ -1404,7 +1482,7 @@ void SlaveBase::dispatchOpenCommand(int command, const QByteArray &data)
     case CMD_NONE:
         break;
     case CMD_CLOSE:
-        close();                // must call finish(), which will set d->inOpenLoop=false
+        close(); // must call finish(), which will set d->inOpenLoop=false
         break;
     default:
         // Some command we don't understand.
@@ -1415,8 +1493,10 @@ void SlaveBase::dispatchOpenCommand(int command, const QByteArray &data)
 
 bool SlaveBase::cacheAuthentication(const AuthInfo &info)
 {
+#ifndef KIO_ANDROID_STUB
     KPasswdServerClient *passwdServerClient = d->passwdServerClient();
     passwdServerClient->addAuthInfo(info, metaData(QStringLiteral("window-id")).toLongLong());
+#endif
     return true;
 }
 
@@ -1476,14 +1556,20 @@ void SlaveBase::setKillFlag()
 
 void SlaveBase::send(int cmd, const QByteArray &arr)
 {
-    slaveWriteError = false;
-    if (!d->appConnection.send(cmd, arr))
+    if (d->runInThread) {
+        if (!d->appConnection.send(cmd, arr)) {
+            exit();
+        }
+    } else {
+        slaveWriteError = false;
+        if (!d->appConnection.send(cmd, arr))
         // Note that slaveWriteError can also be set by sigpipe_handler
-    {
-        slaveWriteError = true;
-    }
-    if (slaveWriteError) {
-        exit();
+        {
+            slaveWriteError = true;
+        }
+        if (slaveWriteError) {
+            exit();
+        }
     }
 }
 
@@ -1491,14 +1577,21 @@ void SlaveBase::virtual_hook(int id, void *data)
 {
     Q_UNUSED(data);
 
-    switch(id) {
+    switch (id) {
     case GetFileSystemFreeSpace: {
         error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_FILESYSTEMFREESPACE));
-    } break;
+        break;
+    }
     case Truncate: {
         error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_TRUNCATE));
-    } break;
+        break;
     }
+    }
+}
+
+void SlaveBase::setRunInThread(bool b)
+{
+    d->runInThread = b;
 }
 
 void SlaveBase::lookupHost(const QString &host)
@@ -1512,7 +1605,7 @@ int SlaveBase::waitForHostInfo(QHostInfo &info)
     QByteArray data;
     int result = waitForAnswer(CMD_HOST_INFO, 0, data);
 
-    if (result  == -1) {
+    if (result == -1) {
         info.setError(QHostInfo::UnknownError);
         info.setErrorString(i18n("Unknown Error"));
         return result;
@@ -1536,24 +1629,25 @@ int SlaveBase::waitForHostInfo(QHostInfo &info)
 
 PrivilegeOperationStatus SlaveBase::requestPrivilegeOperation(const QString &operationDetails)
 {
-
     if (d->m_privilegeOperationStatus == OperationNotAllowed) {
         QByteArray buffer;
         send(MSG_PRIVILEGE_EXEC);
         waitForAnswer(MSG_PRIVILEGE_EXEC, 0, buffer);
         QDataStream ds(buffer);
-        ds >> d->m_privilegeOperationStatus >> d->m_warningCaption >> d-> m_warningMessage;
+        ds >> d->m_privilegeOperationStatus >> d->m_warningTitle >> d->m_warningMessage;
     }
 
-    if (metaData(QStringLiteral("UnitTesting")) != QLatin1String("true") &&
-            d->m_privilegeOperationStatus == OperationAllowed &&
-            !d->m_confirmationAsked) {
-        //KF6 TODO Remove. We don't want to pass details as meta-data. Pass it as a parameter in messageBox().
+    if (metaData(QStringLiteral("UnitTesting")) != QLatin1String("true") && d->m_privilegeOperationStatus == OperationAllowed && !d->m_confirmationAsked) {
+        // WORKER_MESSAGEBOX_DETAILS_HACK
+        // SlaveBase::messageBox() overloads miss a parameter to pass an details argument.
+        // As workaround details are passed instead via metadata before and then cached by the SlaveInterface,
+        // to be used in the upcoming messageBox call (needs WarningContinueCancelDetailed type)
+        // TODO: add a messageBox() overload taking details and use here,
+        // then remove or adapt all code marked with WORKER_MESSAGEBOX_DETAILS
         setMetaData(QStringLiteral("privilege_conf_details"), operationDetails);
         sendMetaData();
 
-        int result = messageBox(d->m_warningMessage, WarningContinueCancelDetailed,
-                                d->m_warningCaption, QString(), QString(), QString());
+        int result = messageBox(d->m_warningMessage, WarningContinueCancelDetailed, d->m_warningTitle, QString(), QString(), QString());
         d->m_privilegeOperationStatus = result == Continue ? OperationAllowed : OperationCanceled;
         d->m_confirmationAsked = true;
     }
@@ -1569,6 +1663,6 @@ void SlaveBase::addTemporaryAuthorization(const QString &action)
 #if KIOCORE_BUILD_DEPRECATED_SINCE(5, 66)
 PrivilegeOperationStatus SlaveBase::requestPrivilegeOperation()
 {
-        return KIO::OperationNotAllowed;
+    return KIO::OperationNotAllowed;
 }
 #endif
