@@ -9,14 +9,13 @@
 #define KIO_SKIPDIALOG_H
 
 #include "kiowidgets_export.h"
-#include <kio/jobuidelegateextension.h>
 #include <QDialog>
+#include <kio/jobuidelegateextension.h>
 
 class QWidget;
 
 namespace KIO
 {
-
 class SkipDialogPrivate;
 /**
  * @internal
@@ -26,7 +25,7 @@ class KIOWIDGETS_EXPORT SkipDialog : public QDialog
     Q_OBJECT
 public:
     SkipDialog(QWidget *parent, KIO::SkipDialog_Options options, const QString &_error_text);
-    ~SkipDialog();
+    ~SkipDialog() override;
 
 private Q_SLOTS:
     void cancelPressed();
@@ -35,7 +34,15 @@ private Q_SLOTS:
     void retryPressed();
 
 Q_SIGNALS:
-    void result(SkipDialog *_this, int _button);
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 79)
+    /**
+     * This signal is overloaded in this class.
+     *
+     * @deprecated since 5.79, Use QDialog::finished(int result)
+     */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 79, "Use QDialog::finished(int result)")
+    void result(SkipDialog *_this, int _button); // clazy:exclude=fully-qualified-moc-types,overloaded-signal
+#endif
 
 private:
     SkipDialogPrivate *const d;
